
#include "producer_consumer.h"

int main() {

  int semid;
  int shmid;
  int removed_item = 0;
  struct sembuf sops[SEM_NUM];
  union semun arg;
  /*--- pointer auf Shared Memory, erhlt man von shmat() ---*/
  prod_con_shm_t *shmPtr;

  /*--- Erzeugung des Shm, falls noch nicht vorhanden,
    Rueckgabe ist die Shared Memory Id (vergleichbar
    mit Dateideskriptor ---*/ 
  if ( 0 > ( shmid = shmget(SHM_KEY,
			    sizeof(prod_con_shm_t),
			    (IPC_CREAT | SHM_PERM)) ) ) {
    perror("consumer: problem with shmget()");
    exit(1);
  }

  /*--- shmat() bildet das Shared Memory in den
    lokalen Adressraum ab, Anfangsadresse
    liegt auf dem Rueckgabewert shmPtr ---*/ 
  if ( ((void*) -1) == (shmPtr = shmat(shmid,0,0)) ) {
    perror("consumer: problem with shmat()");
    exit(1);
  }

  printf("POINTER VALUE shmPtr %p\n", shmPtr);

  /*--- Erzeugen einer Semaphore, falls noch nicht vorhanden.
    Rueckgabe ist wieder ein "Dateideskriptor" ---*/ 
  if ( 0 > (semid = semget(SEM_KEY,
			   SEM_NUM,
			   (IPC_CREAT | SEM_PERM))) ) {
    perror("consumer: problem with semget()");
    exit(1);
  }

  printf("consumer: INITIALISE SEM_MUTEX=1, SEM_FREE = BUFFER_SIZE, SEM_AVAIL = 0\n");
    

  /*--- Der Zhler von MUTEX muss auf 1, der von FREE auf BUFFER_SIZE
    und der von AVAIL auf 0 initialisiert werden.
    ACHTUNG: falls mehrere Consumer aktiv sind, darf nur der erste
    die Initialisierung vornehmen.
    Alternativ knnen alle Semaphoren auch zusammen in einem Schritt
    mit SETALL anstatt einzelnd mit SETVAL gesetzt werden. ---*/ 
  memset(&arg, 0, sizeof(union semun)); 
  arg.val = 1;
  if ( 0 > semctl(semid,SEM_MUTEX,SETVAL,arg) ) {
    perror("consumer: problem with semctl(SEM_MUTEX)");
    exit(1);
  }
  arg.val = BUFFER_SIZE;
  if ( 0 > semctl(semid, SEM_FREE, SETVAL, arg) ) {
    perror("consumer: problem with semctl(SEM_FULL)");
    exit(1);
  }
  arg.val = 0;
  if ( 0 > semctl(semid,SEM_AVAIL,SETVAL,arg) ) {
    perror("consumer: problem with semctl(SEM_AVAIL)");
    exit(1);
  }


  /*--- Consumer Schleife ---*/ 
  while(1) {
    printf("consumer: DOWN AVAIL and DOWN MUTEX\n");


    /*--- DOWN auf MUTEX und AVAIL kann als atomare Operation
      implementiert werden. Alternativ kann man beim
      producer.c sehen, wie die Befehle einzeln abgesetzt 
      werden koennen. ---*/  
    sops[0].sem_num = SEM_MUTEX;
    sops[0].sem_op = -1;
    sops[0].sem_flg = 0;

    sops[1].sem_num = SEM_AVAIL;
    sops[1].sem_op = -1;
    sops[1].sem_flg = 0;
 
    if ( 0 > semop(semid, sops, 2) ) {
      perror("consumer: problem with semop()");
      exit(1);
    }    


    /*--- nimm einen Item vom Stack ---*/ 
    printf("consumer: remove item %d\n",removed_item++);
    printf("consumer: %s\n", 
	   shmPtr->userData[--(shmPtr->numItemsInBuffer)]);
        
    
    printf("consumer: UP MUTEX and UP FULL\n");
    
   
    sops[0].sem_num = SEM_MUTEX;
    sops[0].sem_op = 1;
    sops[0].sem_flg = 0;  

    sops[1].sem_num = SEM_FREE;
    sops[1].sem_op = 1;
    sops[1].sem_flg = 0;  
 
    if ( 0 > semop(semid, sops, 2) ) {
      perror("consumer: problem with semop()");
      exit(1);
    }
  }

  return 0;
}





